﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO.Ports;
using System.IO;
using System.Windows.Forms;
using System.Threading;
using System.Globalization;
using System.Diagnostics;

namespace ISP_BOOT_GUI
{

	public partial class Form1 : Form
	{
		/* ISP_BOOT_GUI activation */
		public Form1()
		{
			InitializeComponent();

			com_start_button.EnabledChanged += Com_start_button_EnabledChanged;
			trans_apli_button.EnabledChanged += Trans_apli_button_EnabledChanged;
			start_apli_button.EnabledChanged += Start_apli_button_EnabledChanged;
			reset_system_button.EnabledChanged += Reset_system_button_EnabledChanged;

			transfer_dialog.Visible = false;
			transfer_dialog.Style = ProgressBarStyle.Continuous;
			rate_label.Visible = false;
			result_rate.Visible = false;
			percent_label.Visible = false;
			percent_label.BackColor = TransparencyKey;

			com_start_button.Enabled = true;
			com_start_button.BackColor = Color.Coral;
			com_port_list.Enabled = true;
			baudrate_list.Enabled = true;
			trans_apli_button.Enabled = false;
			start_apli_button.Enabled = false;
			reset_system_button.Enabled = false;

			comboBox1_SelectedIndexChanged();   /* Get available COM port and add it to the list. */
			comboBox2_SelectedIndexChanged();   /* Default setting of baud rate. */
		}

		public void Form1_Load(object sender, EventArgs e)
		{
		}

		/* Get an array of available COM port names. */
		private void comboBox1_SelectedIndexChanged()
		{
			string[] PortList = SerialPort.GetPortNames();

			com_port_list.Items.Clear();

			/* Set the COM port name in the drop-down list. */
			foreach (string PortName in PortList)
			{
				com_port_list.Items.Add(PortName);
			}

			if (com_port_list.Items.Count > 0)
			{
				com_port_list.SelectedIndex = 0;
			}
			else
			{
				/* Do Nothing */
			}
		}

		/* Set the default of the baud rate drop-down list to 115,200 bps. */
		private void comboBox2_SelectedIndexChanged()
		{
			baudrate_list.SelectedIndex = 10;
		}

		/* Start setting of UART communication. */
		private void com_start_button_Click(object sender, EventArgs e)
		{

			if (serialPort1.IsOpen == true)
			{
				serialPort1.Close();
			}
			else
			{
				/* Do Nothing */
			}

			/* Extract the port to be opened from the COM port list. */
			serialPort1.PortName = com_port_list.SelectedItem.ToString();           /* COM port setting. */
			serialPort1.BaudRate = Convert.ToInt32(baudrate_list.SelectedItem);     /* Baud rate setting. */
			serialPort1.DataBits = 8;                                               /* 8-bit transmission_reception. */
			serialPort1.Parity = Parity.None;                                       /* No parity. */
			serialPort1.StopBits = StopBits.One;                                    /* Stop bit: 1-bit */
			serialPort1.Handshake = Handshake.None;                                 /* No flow control. */
			serialPort1.ReadTimeout = 5000;                                         /* Receive timeout 5 seconds. */
			serialPort1.WriteTimeout = 5000;                                        /* Transmission timeout 5 seconds. */
																					/*
																					serialPort1.Encoding = Encoding.GetEncoding("Shift-JIS");
																					*/

			/* Open COM port. */
			try
			{
				serialPort1.Open();
			}
			catch
			{
				MessageBox.Show("COM Port Open Error.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				serialPort1.Close();
				return;
			}

			/* Send 'x'(0x78). */
			try
			{
				byte[] buffer_s_x = new byte[1];
				char data;
				data = 'x';
				buffer_s_x[0] = (byte)data;
				serialPort1.Write(buffer_s_x, 0, 1);
			}
			catch
			{
				MessageBox.Show("Communication failure.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				serialPort1.Close();
				Close();
			}

			/* Receive 'x'. */
			byte[] buffer_r_x = new byte[1];
			try
			{
				buffer_r_x[0] = (byte)serialPort1.ReadByte();
			}
			catch
			{
				MessageBox.Show("Communication failure.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				serialPort1.Close();
				Close();
			}

			if (buffer_r_x[0] == 'x')
			{

				/* Activate / Deactivate each button. */
				com_start_button.Enabled = false;
				com_port_list.Enabled = false;
				baudrate_list.Enabled = false;
				trans_apli_button.Enabled = true;
				start_apli_button.Enabled = true;
				reset_system_button.Enabled = true;
			}
			else
			{
				MessageBox.Show("Communication failure.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				serialPort1.Close();
				Close();
			}
		}

		/* "Application transfer : 't' " is described at the end. */

		/* "Application start : 's' " input processing. */
		private void start_apli_Click(object sender, EventArgs e)
		{

			/* Send 's'. */
			byte[] buffer_s_s = new byte[1];
			char data;
			data = 's';
			buffer_s_s[0] = (byte)data;
			serialPort1.Write(buffer_s_s, 0, 1);

			/* Receive 's'. */
			byte[] buffer_r_s = new byte[1];
			buffer_r_s[0] = (byte)serialPort1.ReadByte();

			if (buffer_r_s[0] == 's')
			{
				MessageBox.Show("Application launched.", "START", MessageBoxButtons.OK, MessageBoxIcon.Information);

				/* Close the COM port. */
				serialPort1.Close();

				/* Close Form. */
				Close();
			}
			else
			{
				MessageBox.Show("The application could not be started.", "START", MessageBoxButtons.OK, MessageBoxIcon.Error);

			}
		}

		/* "System Reset : 't' " input processing. */
		private void reset_system_Click(object sender, EventArgs e)
		{

			/* Send 'r'. */
			byte[] buffer_s_r = new byte[1];
			char data;
			data = 'r';
			buffer_s_r[0] = (byte)data;
			serialPort1.Write(buffer_s_r, 0, 1);

			/* Receive 'r'. */
			byte[] buffer_r_r = new byte[1];
			buffer_r_r[0] = (byte)serialPort1.ReadByte();

			if (buffer_r_r[0] == 'r')
			{
				MessageBox.Show("Reset the system.", "RESET", MessageBoxButtons.OK, MessageBoxIcon.Information);

				/* Close the COM port. */
				serialPort1.Close();

				/* Activate / Deactivate each button. */
				com_start_button.Enabled = true;
				com_port_list.Enabled = true;
				baudrate_list.Enabled = true;
				trans_apli_button.Enabled = false;
				start_apli_button.Enabled = false;
				reset_system_button.Enabled = false;
			}
			else
			{
				MessageBox.Show("Could not reset system.", "RESET", MessageBoxButtons.OK, MessageBoxIcon.Error);

			}
		}

		/* "Application transfer : 't' " input processing. */
		private void trans_apli_Click(object sender, EventArgs e)
		{
			/* File selection dialog generation. */
			openFileDialog1.Title = "Select Files";
			openFileDialog1.InitialDirectory = "C:\\";
			openFileDialog1.FileName = "";
			openFileDialog1.Filter = "Binary File (*.bin)|*.bin|All Files (*.*)|*.*";
			openFileDialog1.FilterIndex = 1;
			openFileDialog1.RestoreDirectory = true;

			/* File selection dialog display. */
			DialogResult result = openFileDialog1.ShowDialog();
			string FilePath = null;

			if (result == DialogResult.OK)  /* When "Open" button is pressed. */
			{
				FilePath = openFileDialog1.FileName; /* Get the path of the file. */

				/* Activate / Deactivate each button. */
				com_start_button.Enabled = false;
				com_port_list.Enabled = false;
				baudrate_list.Enabled = false;
				trans_apli_button.Enabled = false;
				start_apli_button.Enabled = false;
				reset_system_button.Enabled = false;

				openFileDialog1.Dispose();

			}
			else if (result == DialogResult.Cancel) /* When "Cancel" or "×" button is pressed. */
			{
				openFileDialog1.Dispose();
				return;
			}

			/* Send 't'. */
			byte[] buffer_s_t = new byte[1];
			char data;
			data = 't';
			buffer_s_t[0] = (byte)data;
			serialPort1.Write(buffer_s_t, 0, 1);

			/* Receive 't'. */
			byte[] buffer_r_t = new byte[1];
			buffer_r_t[0] = (byte)serialPort1.ReadByte();

			if (buffer_r_t[0] == 't')
			{
				/* Do Nothing */
			}
			else
			{
				MessageBox.Show("The application could not be transferred.", "TRANSFER", MessageBoxButtons.OK, MessageBoxIcon.Error);

				/* Activate / Deactivate each button. */
				com_start_button.Enabled = false;
				com_port_list.Enabled = false;
				baudrate_list.Enabled = false;
				trans_apli_button.Enabled = true;
				start_apli_button.Enabled = true;
				reset_system_button.Enabled = true;

				openFileDialog1.Dispose();
				return;
			}

			/* Flash Erase confirmation. */
			byte[] buffer_r_ERASE = new byte[1];
			buffer_r_ERASE[0] = (byte)serialPort1.ReadByte();

			/* ACK (0x06 : Normal Response) Response confirmation. */
			if (buffer_r_ERASE[0] == 0x06)
			{
				/* Do Nothing */
			}

			/* NAK (0x15 : Abnormal Response) response confirmation. */
			else
			{
				MessageBox.Show("FLASH Erase failure.", "TRANSFER", MessageBoxButtons.OK, MessageBoxIcon.Error);

				/* Activate / Deactivate each button. */
				com_start_button.Enabled = false;
				com_port_list.Enabled = false;
				baudrate_list.Enabled = false;
				trans_apli_button.Enabled = true;
				start_apli_button.Enabled = true;
				reset_system_button.Enabled = true;

				openFileDialog1.Dispose();
				return;
			}

			/* Start writing to FLASH. */
			byte[] buffer_XMODEM_CRC = new byte[133];
			byte[] buffer_bin = new byte[128];

			/* Receive a transfer start request 'C'(XMODEM/CRC Control-Code) from TARGET. */
			byte[] buffer_r_C = new byte[1];
			buffer_r_C[0] = (byte)serialPort1.ReadByte();

			if (buffer_r_C[0] != 'C')	/* Other than 'C'. */
			{
				MessageBox.Show("Transfer failure.", "TRANSFER", MessageBoxButtons.OK, MessageBoxIcon.Error);

				/* Activate / Deactivate each button. */
				com_start_button.Enabled = false;
				com_port_list.Enabled = false;
				baudrate_list.Enabled = false;
				trans_apli_button.Enabled = true;
				start_apli_button.Enabled = true;
				reset_system_button.Enabled = true;

				openFileDialog1.Dispose();
				return;
			}
			else
			{
				/* Application (Binary File) transfer start. */
				FileStream send_bin = new FileStream(FilePath, FileMode.Open, FileAccess.Read);
				int fileSize = (int)send_bin.Length;    /* Get file size. */
				int readSize;                           /* The number of bytes read by the Read-Method. */
				int remain = fileSize;                  /* The remaining number of bytes to read. */
				byte error_count = 0;                   /* NAK response count. */

				buffer_XMODEM_CRC[0] = 0x01;            /* Always store "SOH"(0x01 : XMODEM/CRC Control-Code) during transfer. */
				byte block_num = 1;                     /* The initial value is 1 according to XMODEM/CRC standard. */

				trans_apli_button.Visible = false;
				transfer_dialog.Visible = true;
				result_rate.Visible = true;
				rate_label.Visible = true;
				percent_label.Visible = true;
				transfer_dialog.SendToBack();
				result_rate.BringToFront();
				percent_label.BringToFront();

				/* Loop until transfer is complete. */
				while (remain > 0)
				{
					ushort rate_value;
					rate_value = (ushort)(100 - (remain * 100 / fileSize));
					transfer_dialog.Value = rate_value;
					result_rate.Text = rate_value.ToString();
					
					/* Block number generation. */
					buffer_XMODEM_CRC[1] = block_num;           /* Store block number. */
					buffer_XMODEM_CRC[2] = (byte)(~block_num);  /* 1's complement storage of block number. */
					if (block_num == 255)
					{
						block_num = 0;                          /* According to XMODEM/CRC standard 255 is 0. */
					}
					else
					{
						block_num++;                            /* Update block number. */
					}

					readSize = send_bin.Read(buffer_bin, 0, Math.Min(128, remain));
					remain -= readSize;

					/* When less than 128-bytes remaining. */
					if (readSize < 128)
					{
						byte rest;
						for (rest = (byte)readSize; rest < 128; rest++)
						{
							/* According to the XMODEM/CRC standard, free space can be filled with "0x1A". */
							buffer_bin[rest] = 0x1a;
						}
						remain = 0;

					}

					/* Data transfer. */
					byte send_num;
					for (send_num = 0; send_num < 128; send_num++)
					{
						buffer_XMODEM_CRC[send_num + 3] = buffer_bin[send_num];
					}

					/* CRC-Code generation. */
					ushort result_crc = calc_crc(buffer_bin);
					buffer_XMODEM_CRC[131] = (byte)(result_crc >> 8);
					buffer_XMODEM_CRC[132] = (byte)(result_crc & 0x00FF);

					/* Data transmission. */
					serialPort1.Write(buffer_XMODEM_CRC, 0, 133);

					/* Receive response. */
					while (error_count < 3) /* End with NAK response 3 or more times. */
					{
						byte response = (byte)serialPort1.ReadByte();
						if (response == 0x15)   /* In case of NAK response (retransmission request). */
						{
							error_count++;
							if (error_count >= 3)
							{
								MessageBox.Show("Failed to write to FLASH.", "TRANSFER", MessageBoxButtons.OK, MessageBoxIcon.Error);
								/* Activate / Deactivate each button. */
								com_start_button.Enabled = false;
								com_port_list.Enabled = false;
								baudrate_list.Enabled = false;
								trans_apli_button.Enabled = true;
								start_apli_button.Enabled = true;
								reset_system_button.Enabled = true;

								trans_apli_button.Visible = true;
								transfer_dialog.Visible = false;
								result_rate.Visible = false;
								rate_label.Visible = false;
								percent_label.Visible = false;

								send_bin.Dispose();
								return;

							}
							else
							{
								/* Do Nothing */
							}

							/* Send the same block again. */
							serialPort1.Write(buffer_XMODEM_CRC, 0, 133);
						}
						else if (response == 0x06)  /* In case of ACK response (next block request). */
						{
							break;
						}
					}
				}

				/* Finally send "EOT"(XMODEM/CRC Control-Code). */
				byte[] buffer_s_EOT = new byte[1];
				buffer_s_EOT[0] = 0x04;
				serialPort1.Write(buffer_s_EOT, 0, 1);

				/* Receive the ACK response of transfer end and transfer complete. */
				byte[] buffer_r_EOT = new byte[1];
				buffer_r_EOT[0] = (byte)serialPort1.ReadByte();

				transfer_dialog.Visible = false;
				result_rate.Visible = false;
				rate_label.Visible = false;
				percent_label.Visible = false;
				trans_apli_button.Visible = true;

				/* Receive ACK response of write complete from TARGET and transfer process completed. */
				byte[] buffer_r_ACK = new byte[1];
				buffer_r_ACK[0] = (byte)serialPort1.ReadByte();
				if (buffer_r_ACK[0] == 0x06)
				{
					MessageBox.Show("Write successfully to FLASH !!", "TRANSFER", MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
				else
				{
					MessageBox.Show("Failed to write to FLASH.", "TRANSFER", MessageBoxButtons.OK, MessageBoxIcon.Error);

				}

				/* Activate / Deactivate each button. */
				com_start_button.Enabled = false;
				com_port_list.Enabled = false;
				baudrate_list.Enabled = false;
				trans_apli_button.Enabled = true;
				start_apli_button.Enabled = true;
				reset_system_button.Enabled = true;

				send_bin.Dispose();
				return;

				/* CRC-Code calculation function. */
				ushort calc_crc(byte[] buf)
				{
					ushort crc;
					var flag = 0;
					byte i, j;

					crc = 0;
					for (i = 0; i < 128; i++)
					{
						crc ^= (ushort)(buf[i] << 8);
						for (j = 0; j < 8; j++)
						{
							flag = crc & 0x8000;
							if (flag != 0)
							{
								crc = (ushort)((crc << 1) ^ 0x1021);
							}
							else
							{
								crc = (ushort)(crc << 1);
							}
						}
					}
					return crc;
				}
				/* The original function of C-language. */
				//				static uint16_t calc_crc(void)
				//				{
				//					uint16_t crc = 0x0000U;
				//					uint8_t i, j;
				//					for (i = 3; i < (WRITE_SIZE + 3); i++) {
				//						crc ^= (uint16_t)Recvbuf[i] << 8;
				//						for (j = 0; j < 8; j++) {
				//						if (crc & 0x8000U) {
				//							 crc = (crc << 1) ^ 0x1021U;
				//						} else {
				//							crc <<= 1;
				//							}
				//						}
				//					}
				//					return crc;
				//				}
			}
		}

		/* Change the background color of each button.*/
		private void Com_start_button_EnabledChanged(object sender, EventArgs e)
		{
			bool status;
			status = com_start_button.Enabled;

			if (status == true)
			{
				com_start_button.BackColor = Color.Coral;
			}
			else
			{
				com_start_button.BackColor = Color.Gray;

			}
		}
		private void Trans_apli_button_EnabledChanged(object sender, EventArgs e)
		{
			bool status;
			status = trans_apli_button.Enabled;

			if (status == true)
			{
				trans_apli_button.BackColor = Color.CornflowerBlue;
			}
			else
			{
				trans_apli_button.BackColor = Color.Gray;
			}
		}
		private void Start_apli_button_EnabledChanged(object sender, EventArgs e)
		{
			bool status;
			status = start_apli_button.Enabled;

			if (status == true)
			{
				start_apli_button.BackColor = Color.Gold;
			}
			else
			{
				start_apli_button.BackColor = Color.Gray;

			}
		}
		private void Reset_system_button_EnabledChanged(object sender, EventArgs e)
		{
			bool status;
			status = reset_system_button.Enabled;

			if (status == true)
			{
				reset_system_button.BackColor = Color.LimeGreen;
			}
			else
			{
				reset_system_button.BackColor = Color.Gray;

			}
		}


		/* no use */
		private void start_message_Click(object sender, EventArgs e)
		{
		}
		private void com_input_Click(object sender, EventArgs e)
		{
		}
		private void label1_Click(object sender, EventArgs e)
		{
		}
		private void label1_Click_1(object sender, EventArgs e)
		{
		}
		private void label4_Click(object sender, EventArgs e)
		{
		}
		private void textBox1_TextChanged(object sender, EventArgs e)
		{
		}
		private void label2_Click(object sender, EventArgs e)
		{
		}
	}/* End of Form1 */
}
