/** 
 *******************************************************************************
 * @file    Flash_Erase.c
 * @brief   ISP_BOOT Flash_Erase(PG2-PG7,Block1-Block15) Sample Application.
 * @version V1.1.0.0
 * $Date:: 2018-11-29
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2018 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif


/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "stdio.h"

#include "system_TMPM3Hy.h"
#include "TMPM3HQ.h"

#include "txz_uart.h"
#include "txz_t32a.h"

#include "main.h"
#include "com_control.h"
#include "Flash_Erase.h"
#include "Flash_Write.h"


#if defined ( __CC_ARM )        /* RealView Compiler */
extern uint32_t Load$$FLASH_ERASE_RAM$$Base;
extern uint32_t Image$$FLASH_ERASE_RAM$$Base;
extern uint32_t Load$$FLASH_ERASE_RAM$$RO$$Length;

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#endif

/*------------------------------------------------------------------------------*/
/*  Function Definitions                                                        */
/*------------------------------------------------------------------------------*/
uint8_t Flash_Erase(void);
void Erase_IPRT(void);
void Erase_Code_Flash(void);
void Erase_ISEC(void);
uint8_t Blank_Check_IPRT(void);
uint8_t Blank_Check_CodeFlash(void);
uint8_t Blank_Check_ISEC(void);
void Get_RDY_Status(void);
void Get_BSY_Status(void);

void Select_AREA0(void);
void Deselect_AREA0(void);

/*------------------------------------------------------------------------------*/
/*  Variable Definitions                                                        */
/*------------------------------------------------------------------------------*/
volatile uint32_t *prt_FLASH_SEQ_1st;	/* First-Bus-Cycle address. */
volatile uint32_t *prt_FLASH_SEQ_2nd;	/* Second-Bus-Cycle address. */
volatile uint32_t *prt_Erase_Addr_Top;	/* Erase start address. */


/*--------------------------------------------------*/
/** 
  * @brief  Flash Erase Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint8_t Flash_Erase(void)
{

	uint8_t error_blank = 0;

	/* AREA-0 selection. */
	Select_AREA0();

	/* Code flash protect bit erase.(PG2-PG7,Block1-Block15) */
	Erase_IPRT();		/* Infinite loop continuing during FLASH operation. */

	/* Code flash erase.(PG2-PG7,Block1-Block15) */
	Erase_Code_Flash();	/* Infinite loop continuing during FLASH operation. */

	/* Security bit erase. */
	Erase_ISEC();		/* Infinite loop continuing during FLASH operation. */

	/* AREA-0 release. */
	Deselect_AREA0();


	/* Prefetch buffer disabled and cleared. */
	do {
		TSB_FC->BUFDISCLR = (0x00000007UL);
	} while (TSB_FC->BUFDISCLR != (0x00000007UL));	/* Read confirmation after writing. */

	/* Prefetch buffer permission. */
	do {
		TSB_FC->BUFDISCLR = (0x00000000UL);
	} while (TSB_FC->BUFDISCLR != (0x00000000UL));	/* Read confirmation after writing. */


	/* Code flash protection bit blank check.(PG2-PG7,Block1-Block15) */
	error_blank = Blank_Check_IPRT();
	if (error_blank == 1) {
		return 1;
	} else {
		/* Do Nothing */
	}

	/* Code flash blank check.(PG2-PG7,Block1-Block15) */
	Blank_Check_CodeFlash();
	if (error_blank == 1) {
		return 1;
	} else {
		/* Do Nothing */
	}

	/* Security bit blank check. */
	Blank_Check_ISEC();
	if (error_blank == 1) {
		return 1;
	} else {
		/* Do Nothing */
	}

	return 0;

}


/*--------------------------------------------------*/
/** 
  * @brief  Code flash protect bit erase.(PG2-PG7,Block1-Block15)
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Erase_IPRT(void)
{

	/* Check FLASH Status */
	Get_RDY_Status();		/* Infinite loop continuing during FLASH operation. */

	/* Bus-Cycle address and Erase start address setting. */
	prt_FLASH_SEQ_1st = (uint32_t *)(CODEF_BGN + FLASH_SEQ_1ST);
	prt_FLASH_SEQ_2nd = (uint32_t *)(CODEF_BGN + FLASH_SEQ_2ND);
	prt_Erase_Addr_Top  = (uint32_t *)(CODEF_BGN + CODEF_IPRT_OFFSET);

	/* Auto protect bit erase command. */
	*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
	*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
	*prt_FLASH_SEQ_1st = FLASH_CMD_80;
	*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
	*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
	*prt_Erase_Addr_Top = FLASH_CMD_60;	/* PBA */

	/* Check FLASH Status */
	Get_RDY_Status();		/* Infinite loop continuing during FLASH operation. */

	/* PG2 - PG7 protect mask */
	TSB_FC->KCR = KEYCODE;			/* Key code input. */
	TSB_FC->PMR0 = (0xFFFFFF00UL);

	/* Block1 - Block15 protect mask */
	TSB_FC->KCR = KEYCODE;			/* Key code input. */
	TSB_FC->PMR1 = (0xFFFF0000UL);

	return;
}


/*--------------------------------------------------*/
/** 
  * @brief  Code flash erase.(PG2-PG7,Block1-Block15)
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Erase_Code_Flash(void)
{

	uint8_t i;

	/* Check FLASH Status */
	Get_RDY_Status();		/* Infinite loop continuing during FLASH operation. */

	/* Bus-Cycle address and Erase start address setting. */
	prt_FLASH_SEQ_1st = (uint32_t *)(CODEF_BGN + FLASH_SEQ_1ST);
	prt_FLASH_SEQ_2nd = (uint32_t *)(CODEF_BGN + FLASH_SEQ_2ND);
	prt_Erase_Addr_Top  = (uint32_t *)(CODEF_BGN + CODEF_PG2_OFFSET);

	/* PG2 - PG7 erase. */
	for (i = 0 ; i < 6 ; i++) {
		/* Auto page erase command. */
		*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
		*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
		*prt_FLASH_SEQ_1st = FLASH_CMD_80;
		*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
		*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
		*prt_Erase_Addr_Top = FLASH_CMD_40;	/* PGA */

		/* Check FLASH Status */
		Get_RDY_Status();		/* Infinite loop continuing during FLASH operation. */

		/* Page refresh. */
		prt_Erase_Addr_Top  = prt_Erase_Addr_Top + ((0x00001000UL) / 4);
	}


	/* Update Erase start address. */
	prt_Erase_Addr_Top  = (uint32_t *)(CODEF_BGN + CODEF_BLK1_OFFSET);

	/* Block1 - Block15 erase. */
	for (i = 0 ; i < 15 ; i++) {
		/* Auto block erase command. */
		*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
		*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
		*prt_FLASH_SEQ_1st = FLASH_CMD_80;
		*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
		*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
		*prt_Erase_Addr_Top = FLASH_CMD_30;	/* PGA */

		/* Check FLASH Status */
		Get_RDY_Status();		/* Infinite loop continuing during FLASH operation. */

		/* Block refresh. */
		prt_Erase_Addr_Top  = prt_Erase_Addr_Top + ((0x00008000UL) / 4);
	}


	return;

}


/*--------------------------------------------------*/
/** 
  * @brief  Security bit erase.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Erase_ISEC(void)
{

	/* Check FLASH Status */
	Get_RDY_Status();		/* Infinite loop continuing during FLASH operation. */

	/* Security temporary cancellation. */
	TSB_FC->KCR = KEYCODE;
	TSB_FC->SBMR = ISEC_MASK;

	/* Bus-Cycle address and Erase start address setting. */
	prt_FLASH_SEQ_1st = (uint32_t *)(CODEF_BGN + FLASH_SEQ_1ST);
	prt_FLASH_SEQ_2nd = (uint32_t *)(CODEF_BGN + FLASH_SEQ_2ND);
	prt_Erase_Addr_Top  = (uint32_t *)(CODEF_BGN + ISEC_OFFSET);

	/* Auto security bit erase command. */
	*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
	*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
	*prt_FLASH_SEQ_1st = FLASH_CMD_80;
	*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
	*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
	*prt_Erase_Addr_Top = FLASH_CMD_60;	/* SBA */

	/* Check FLASH Status */
	Get_RDY_Status();		/* Infinite loop continuing during FLASH operation. */

	/* Security settings. */
	TSB_FC->KCR = KEYCODE;
	TSB_FC->SBMR = ISEC_RELEASE;

	return;
}


/*--------------------------------------------------*/
/** 
  * @brief  Code flash protection bit blank check.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint8_t Blank_Check_IPRT(void)
{

	/* Blank Check PG2 - PG7 */
	if ((TSB_FC->PSR0 & 0x000000FF) != 0x00000000) {
		return 1;
	} else {
		/* Do Nothing */
	}

	/* Blank Check Block1 - Block15 */
	if ((TSB_FC->PSR1 & 0x0000FFFE) != 0x00000000) {
		return 1;
	} else {
		/* Do Nothing */
	}

	return 0;	/* Protect bits is Blank */

}


/*--------------------------------------------------*/
/** 
  * @brief  Code flash blank check.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint8_t Blank_Check_CodeFlash(void)
{

	uint32_t *readaddr;

	/* PG2 address setting. */
	readaddr = (uint32_t *)(CODEF_BGN + CODEF_PG2_OFFSET);

	/* PG2 - PG7 , Block1 - Block15 blank check. */
	while (readaddr < (uint32_t *)CODEF_END) {
		if (*readaddr != (0xFFFFFFFFUL)) {
			return 1;
		} else {
			readaddr++;
		}
	}

	return 0;	/* PG2 - PG7 and Block1 - Block15 are Blank. */

}


/*--------------------------------------------------*/
/** 
  * @brief  Security bit blank check.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint8_t Blank_Check_ISEC(void)
{

	/* Blank Check Security bit */
	if ((TSB_FC->SSR & 0x00000001) != 0x00000000) {
		return 1;
	} else {
		/* Do Nothing */
	}

	return 0;	/* Security bit is Blank. */

}


/*--------------------------------------------------*/
/** 
  * @brief  Get Flash Status "Ready".
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Get_RDY_Status(void)
{

	__DSB();			/* Barrier command. */

	/* Infinite loop continuing during FLASH operation. */
	while ((TSB_FC->SR0 & 0x00000001) == (0x00000000UL)) {;}

	return;
}


/*--------------------------------------------------*/
/** 
  * @brief  Get Flash Status "Busy".
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Get_BSY_Status(void)
{

	__DSB();			/* Barrier command. */

	/* Infinite loop while FLASH is stopped. */
	while ((TSB_FC->SR0 & 0x00000001) == (0x00000001UL)) {;}

	return;
}


/*--------------------------------------------------*/
/** 
  * @brief  AREA-0 selection.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Select_AREA0(void)
{


	TSB_FC->KCR = KEYCODE;				/* Key code input. */
	TSB_FC->AREASEL = 0x00000007;		/* AREA-0 selection. */
	while ((TSB_FC->AREASEL & 0x04000000) != (0x04000000UL)) {;}	/* Confirm AREA-0 selection. */

	return;
}


/*--------------------------------------------------*/
/** 
  * @brief  AREA-0 release.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Deselect_AREA0(void)
{


	TSB_FC->KCR = KEYCODE;				/* Key code input. */
	TSB_FC->AREASEL = (0x00000000UL);	/* AREA-0 release. */
	while ((TSB_FC->AREASEL & 0x04000000) == 0x04000000UL) {;}	/* AREA-0 cancellation confirmation. */

	return;
}


#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif


/**
 *  @} 
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */

