/** 
 *******************************************************************************
 * @file    Flash_Erase.c
 * @brief   ISP_BOOT Flash_Write(PG2-PG7,Block1-Block15) Sample Application.
 * @version V1.1.0.0
 * $Date:: 2018-11-29
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2018 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif


/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "stdio.h"

#include "system_TMPM3Hy.h"
#include "TMPM3HQ.h"

#include "txz_uart.h"
#include "txz_t32a.h"

#include "main.h"
#include "com_control.h"
#include "Flash_Erase.h"
#include "Flash_Write.h"


#if defined ( __CC_ARM )        /* RealView Compiler */
extern uint32_t Load$$FLASH_WRITE_RAM$$Base;
extern uint32_t Image$$FLASH_WRITE_RAM$$Base;
extern uint32_t Load$$FLASH_WRITE_RAM$$RO$$Length;

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#endif



/*------------------------------------------------------------------------------*/
/*  Function Definitions                                                        */
/*------------------------------------------------------------------------------*/
uint8_t Flash_Write(void);

void Write_RecvData(void);
static uint16_t calc_crc(void);
void SendACK(void);
void SendNAK(void);

void BOOT_boot(void);

#if defined ( __CC_ARM )        /* RealView Compiler */
	__ASM void BOOT_jump(void);
#elif defined ( __ICCARM__ )    /* IAR Compiler */
	void BOOT_jump(uint32_t sp, uint32_t pc);
#endif

/*------------------------------------------------------------------------------*/
/*  Variable Definitions                                                        */
/*------------------------------------------------------------------------------*/
extern volatile uint32_t *prt_FLASH_SEQ_1st;	/* First-Bus-Cycle address. */
extern volatile uint32_t *prt_FLASH_SEQ_2nd;	/* Second-Bus-Cycle address. */
	   volatile uint32_t *prt_Write_Addr_Top;	/* Write start address. */

uint8_t Recvbuf[1 + 1 + 1 + 128 + 2];			/* Receive data buffer. */
	/* BreakdownF
		1.Header (1byteFSOH(0x01))
		2.Data block number (1byte)
		3.One's complement of data block number (1byte)
			(Add 2. and 3. to 0xFF.)
		4.Application data (128byte)
		5.CRC-code (2byte)
	*/

uint8_t Sendbuf;		/* Transmit data buffer. */

uint8_t error_count;	/* Communication error counter. */


/*--------------------------------------------------*/
/** 
  * @brief  Flash Write Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint8_t Flash_Write(void)
{

	uint32_t RecvLen = 0;	/* Number of received data bytes. */
	uint16_t crcbuf1;		/* Store the received CRC-code. */
	uint16_t crcbuf2;		/* Store the result of calculating the CRC-code. */

	uint32_t i;


	/* Error counter initialization. */
	error_count = 0;

	/* Receive buffer initialization. */
	for (i = 0; i < DATA_LEN; i++) {
		Recvbuf[i] = 0;
	}

	/* Bus-Cycle address and Write start address setting. */
	prt_FLASH_SEQ_1st  = (uint32_t *)(CODEF_BGN + FLASH_SEQ_1ST);
	prt_FLASH_SEQ_2nd  = (uint32_t *)(CODEF_BGN + FLASH_SEQ_2ND);
	prt_Write_Addr_Top = (uint32_t *)(CODEF_BGN + WRITE_BG_ADDR);


	/* Wait until preparation for application transfer preparation is completed. */
	/* (If it is unnecessary, it may be deleted.) */
	for (i = 0; i < 0x00010000; i++){;}


	/* Send 'C' to HOST and start application transfer. */
	Sendbuf = 'C';
	SendData(&Sendbuf , 1);

	/* Loop until transfer completion or until an error is detected. */
	for (;;) {
		/* Error determination : Returning to command reception with detection of an error three or more times. */
		if (error_count >= 3) {
			return 1;
		} else {
			/* Do Nothing */
		}

		/* Application block reception. */
		RecvLen = RecvData(Recvbuf , DATA_LEN);

		/* Except specified byte reception. */
		if (RecvLen != DATA_LEN) {
			/* Confirm headerF'EOT' -> Transfer end. */
			if (Recvbuf[0] == EOT) {
				/* In the case of transfer end, it exits the loop after sending ACK. */
				SendACK();
				break;

			} else {
				/* Send a retransmission request to HOST. -> Receive same block again. */
				SendNAK();
			}

		/* Received specified byte. */
		} else {
			/* Confirm header */
			if (Recvbuf[0] != SOH) {		/* Other than 'SOH'. */
				SendNAK();					/* Send a retransmission request to HOST. -> Receive same block again. */

			/* Check block number. */
			} else if ((Recvbuf[1] + Recvbuf[2]) != 0xFF) {	/* In case of addition other than 0xFF. */
				SendNAK();					/* Send a retransmission request to HOST. -> Receive same block again. */

			/* CRC-code verification. */
			} else {
				/* CRC-code attached to received data. -> crcbuf1(Big endian) */
				crcbuf1 = (((uint16_t)Recvbuf[131] << 8) | (uint16_t)Recvbuf[132]);
				/* Calculate the CRC-code from the received application data. -> crcbuf2 */
				crcbuf2 = calc_crc();

				/* CRC-code match confirmation. */
				if (crcbuf1 != crcbuf2) {	/* unmatch */
					SendNAK();				/* Send a retransmission request to HOST. -> Receive same block again. */

				} else {					/* match */
					Write_RecvData();		/* Write application data to FLASH. */
					SendACK();				/* After writing, send ACK to HOST. -> Next block request. */

				}
			}
		}
	}

	return 0;

}


/*--------------------------------------------------*/
/** 
  * @brief  Write Receive Program Data on Flash.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Write_RecvData(void)
{

	uint8_t i,j;
	uint32_t write_buf;

	/* AREA-0 selection. */
	Select_AREA0();

	/* Check FLASH Status */
	Get_RDY_Status();

	for (i = 0; i < WRITE_SIZE; i += 16) {

		*prt_FLASH_SEQ_1st = FLASH_CMD_AA;
		*prt_FLASH_SEQ_2nd = FLASH_CMD_55;
		*prt_FLASH_SEQ_1st = FLASH_CMD_A0;

		/* With automatic program command, 16-bytes can be written per command input. */
		for (j = 0; j < 16; j += 4) {
			write_buf = 0;
			write_buf  =  (((uint32_t)Recvbuf[i + j + 3 + 0])        & 0x000000FF);
			write_buf |= ((((uint32_t)Recvbuf[i + j + 3 + 1]) <<  8) & 0x0000FF00);
			write_buf |= ((((uint32_t)Recvbuf[i + j + 3 + 2]) << 16) & 0x00FF0000);
			write_buf |= ((((uint32_t)Recvbuf[i + j + 3 + 3]) << 24) & 0xFF000000);

			*prt_Write_Addr_Top = write_buf;
		}

		/* Check FLASH Status */
		Get_BSY_Status();	/* Wait until the busy state is reached. */
		Get_RDY_Status();	/* Wait until it gets ready. */

		/* Because 16-bytes are automatically written, the address is also updated in units of 16-bytes. */
		prt_Write_Addr_Top += 4;

	}

	/* AREA-0 release. */
	Deselect_AREA0();

	return;
}


/*--------------------------------------------------*/
/** 
  * @brief  Send 'ACK' to HOST.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void SendACK(void)
{

	Sendbuf = ACK;
	SendData(&Sendbuf , 1);

	return;
}


/*--------------------------------------------------*/
/** 
  * @brief  Send 'NAK' to HOST.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void SendNAK(void)
{

	Sendbuf = NAK;
	SendData(&Sendbuf , 1);
	error_count++;

	return;
}


/**************************************************************************
 * @brief This function sets up the Cortex M-3 with a new SP and PC.
 **************************************************************************/
#if defined ( __CC_ARM )        /* RealView Compiler */

__ASM void BOOT_jump(void)
{

	MOV32 R0  , #0x00002000
	LDR   R0  , [R0]
	MSR   MSP , R0
	MSR   PSP , R0

	MOV32 R1  , #0x00002004
	LDR   R1  , [R1]
	BLX   R1

}

#elif defined ( __ICCARM__ )    /* IAR Compiler */

void BOOT_jump(uint32_t sp, uint32_t pc)
{
	(void) sp;
	void (* Jump) (void);

	/* Set new MSP, PSP based on SP (r0)*/
	__asm("msr msp, r0");
	__asm("msr psp, r0");

	/* Jump to PC */
	Jump = (void (*)())pc;
	Jump();

}
#endif

/**************************************************************************
 * @brief Boots the application
 **************************************************************************/
void BOOT_boot(void)
{
#if defined ( __CC_ARM )        /* RealView Compiler */

	uint32_t *VTOR;

	/* Set Vector Offset Register */
	VTOR = (uint32_t *)(0xE000ED08UL);
	*VTOR = (0x00002000UL);

	BOOT_jump();


#elif defined ( __ICCARM__ )    /* IAR Compiler */

	uint32_t pc, sp;
	uint32_t *VTOR , *pc_addr , *sp_addr;

	/* Set Vector Offset Register */
	VTOR = (uint32_t *)(0xE000ED08UL);
	*VTOR = (0x00002000UL);

	/* Read new SP and PC from vector table */
	sp_addr = (uint32_t *)(0x00002000UL);
	sp = *sp_addr;

	pc_addr = (uint32_t *)(0x00002004UL);
	pc = *pc_addr;

	BOOT_jump(sp, pc);

#endif

}


/*--------------------------------------------------*/
/** 
  * @brief  Caluculation CRC.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static uint16_t calc_crc(void)
{

	uint16_t crc = 0x0000U;
	uint8_t i, j;

	for (i = 3; i < (WRITE_SIZE + 3); i++) {
		crc ^= (uint16_t)Recvbuf[i] << 8;
		for (j = 0; j < 8; j++) {
			if (crc & 0x8000U) {
				 crc = (crc << 1) ^ 0x1021U;
			} else {
				crc <<= 1;
			}
		}
	}
	return crc;

}


#if 0
/* 
 Explanation by the developer.
 There are multiple CRC-16 calculations as follows.
 The algorithm slightly changes under conditions such as CRC initial value and shift.
 Customization is necessary according to the intended use.
 ECRC-16/CCITT-FALSE
 ECRC-16/ARC
 ECRC-16/AUG-CCITT
 ECRC-16/BUYPASS
 ECRC-16/CDMA2000
 ECRC-16/DDS-110
 ECRC-16/DECT-R
 ECRC-16/DECT-X
 ECRC-16/DNP
 ECRC-16/EN-13757
 ECRC-16/GENIBUS
 ECRC-16/MAXIM
 ECRC-16/MCRF4XX
 ECRC-16/RIELLO
 ECRC-16/T10-DIF
 ECRC-16/TELEDISK
 ECRC-16/TMS37157
 ECRC-16/USB
 ECRC-A
 ECRC-16/KERMIT
 ECRC-16/MODBUS
 ECRC-16/X-25
 ECRC-16/XMODEM
*/
#endif


/**
 *  @} 
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */


